function [K, Y, numKernels] = createToyDataAndComputeKernels (dataType, visualize)
% dataType = 0: Linear
% dataType = 1: Nonlinear


% Params:
    gamma = 0.5;
   
addpath 'drtoolbox';
addpath 'drtoolbox/techniques';

    numFeatures = 50;

    if (dataType == 0)
        [X, Y] = generate_data(100, 20, numFeatures, false, 0, 0, -1); Y = Y';
    else
        [X, Y] = generate_data_nl(100, numFeatures); Y = Y';
    end
    if (visualize)
        
        close all;
        scatter (X(1,Y==1),X(2,Y==1), 60,...
                  'MarkerEdgeColor',[0 .5 .5],...
                  'MarkerFaceColor',[0 0 1],...
                  'LineWidth',1.5);
        hold on;
        scatter (X(1,Y==-1),X(2,Y==-1), 60,...
                  'MarkerEdgeColor',[0 .5 .5],...
                  'MarkerFaceColor',[1 0 0],...
                  'LineWidth',1.5);

        mX = compute_mapping(X', 'tSNE', 2)';	


        close all;
        scatter (mX(1,Y==1),mX(2,Y==1), 60,...
                  'MarkerEdgeColor',[0 .5 .5],...
                  'MarkerFaceColor',[0 0 1],...
                  'LineWidth',1.5);
        hold on;
        scatter (mX(1,Y==-1),mX(2,Y==-1), 60,...
                  'MarkerEdgeColor',[0 .5 .5],...
                  'MarkerFaceColor',[1 0 0],...
                  'LineWidth',1.5);
    end
    
    % Normalizing the features all to the range [0, 1]
    for i = 1:size(X,1)
        mm = min(X(i,:));
        MM = max(X(i,:));
        X(i,:) = (X(i,:) - mm) / (MM - mm);
    end

  numKernels = 5*numFeatures;
  K = zeros(size(X,2),size(X,2),numKernels);
  
  K(:,:,1:numFeatures                 ) = LINprecomp    (X, X           );
  K(:,:,numFeatures+1:2*numFeatures   ) = EMDprecomp    (X, X, gamma    );
  K(:,:,2*numFeatures+1:3*numFeatures ) = HIKprecomp	(X, X, gamma, 1 );
  K(:,:,3*numFeatures+1:4*numFeatures ) = Chi2precomp   (X, X, gamma    );
  K(:,:,4*numFeatures+1:5*numFeatures ) = EUCprecomp    (X, X, gamma    );
end

function D=EUCprecomp(TRNfeatures,TSTfeatures,gamma)
  [NUMk,NUMtrain]=size(TRNfeatures);
  [NUMk,NUMtest]=size(TSTfeatures);
  D=zeros(NUMtrain,NUMtest,NUMk);
  for k=1:NUMk
      D(:,:,k)=exp(-gamma*pdist2(TRNfeatures(k,:)',TSTfeatures(k,:)','euclidean'));
  end;        
end

function D=EMDprecomp(TRNfeatures,TSTfeatures,gamma)
  [NUMk,NUMtrain]=size(TRNfeatures);
  [NUMk,NUMtest]=size(TSTfeatures);
  D=zeros(NUMtrain,NUMtest,NUMk);
  for k=1:NUMk
      D(:,:,k)=exp(-gamma*pdist2(TRNfeatures(k,:)',TSTfeatures(k,:)','emd'));
  end;        
end

function D=Chi2precomp(TRNfeatures,TSTfeatures,gamma)
  [NUMk,NUMtrain]=size(TRNfeatures);
  [NUMk,NUMtest]=size(TSTfeatures);
  D=zeros(NUMtrain,NUMtest,NUMk);
  for k=1:NUMk
      D(:,:,k)=exp(-gamma*pdist2(TRNfeatures(k,:)',TSTfeatures(k,:)','chisq'));
  end;        
end

function D=LINprecomp(TRNfeatures,TSTfeatures)
  [NUMk,NUMtrain]=size(TRNfeatures);
  [NUMk,NUMtest]=size(TSTfeatures);
  D=zeros(NUMtrain,NUMtest,NUMk);
  for k=1:NUMk
%       x1 = TRNfeatures(k,:); x1 = x1(:);
%       x2 = TSTfeatures(k,:); x2 = x2(:);
      D(:,:,k)=TRNfeatures(k,:)'*TSTfeatures(k,:);
  end;        
end

function D=HIKprecomp(TRNfeatures,TSTfeatures,gamma,beta)
  [NUMk,NUMtrain]=size(TRNfeatures);
  [NUMk,NUMtest]=size(TSTfeatures);
  D=zeros(NUMtrain,NUMtest,NUMk);
  for k=1:NUMk
      D(:,:,k)=bsxfun(@min,abs(repmat(TRNfeatures(k,:)',1,NUMtest)).^beta, abs(repmat(TSTfeatures(k,:),NUMtrain,1)).^beta); 
  end;        
end

function [X,Y] = generate_data_nl(n, D)

    XX = [];
    Y = [];
    for i = 1:D
        XX = [XX; normrnd(1,0.5,[1,n])];
    end
    X = XX;
    Y = [Y,-1*ones(1,n)];    
    
    XX = [];
    for i = 1:D
        XX = [XX; normrnd(1,0.5,[1,n])];
    end
    
    for i = 1:D
        for j = 1:n
            k = -1;
            if (rand() < 0.5)
                k = 1;
            end
          XX (i,j) = XX (i,j) + k*2;
        end
    end
    X = [X, XX];
    Y = [Y,+1*ones(1,n)];    

end